from fastapi import APIRouter

from model_server.base.session import session

from model_server.extensions.ilastik import models as ilm

router = APIRouter(
    prefix='/ilastik',
    tags=['ilastik'],
)


import model_server.extensions.ilastik.pipelines.px_then_ob
router.include_router(model_server.extensions.ilastik.pipelines.px_then_ob.router)

@router.put('/seg/load/')
def load_px_model(p: ilm.IlastikPixelClassifierParams) -> dict:
    """
    Load an ilastik pixel classifier model from its project file
    """
    return load_ilastik_model(
        ilm.IlastikPixelClassifierModel,
        p,
    )

@router.put('/pxmap_to_obj/load/')
def load_pxmap_to_obj_model(p: ilm.IlastikParams) -> dict:
    """
    Load an ilastik object classifier from pixel predictions model from its project file
    """
    return load_ilastik_model(
        ilm.IlastikObjectClassifierFromPixelPredictionsModel,
        p,
    )

@router.put('/seg_to_obj/load/')
def load_seg_to_obj_model(p: ilm.IlastikParams) -> dict:
    """
    Load an ilastik object classifier from segmentation model from its project file
    """
    return load_ilastik_model(
        ilm.IlastikObjectClassifierFromSegmentationModel,
        p,
    )

def load_ilastik_model(model_class: ilm.IlastikModel, p: ilm.IlastikParams) -> dict:
    project_file = p.project_file
    if not p.duplicate:
        existing_model_id = session.find_param_in_loaded_models('project_file', project_file, is_path=True)
        if existing_model_id is not None:
            session.log_info(f'An ilastik model from {project_file} already existing exists; did not load a duplicate')
            return {'model_id': existing_model_id}
    result = session.load_model(model_class, key=p.model_id, params=p)
    session.log_info(f'Loaded ilastik model {result} from {project_file}')
    return {'model_id': result}