from pathlib import Path
import unittest

import numpy as np

from model_server.base.roiset import RoiSetWithDerivedChannelsExportParams, RoiSetMetaParams
from model_server.base.roiset import RoiSetWithDerivedChannels
from model_server.base.accessors import generate_file_accessor, PatchStack
import model_server.conf.testing as conf
from tests.base.test_model import DummyInstanceSegmentationModel

data = conf.meta['image_files']
params = conf.meta['roiset']
output_path = conf.meta['output_path']

class TestDerivedChannels(unittest.TestCase):
    def setUp(self) -> None:
        self.stack = generate_file_accessor(data['multichannel_zstack_raw']['path'])
        self.stack_ch_pa = self.stack.get_mono(params['patches_channel'])
        self.seg_mask = generate_file_accessor(data['multichannel_zstack_mask2d']['path'])

    def test_classify_by_with_derived_channel(self):
        class ModelWithDerivedInputs(DummyInstanceSegmentationModel):
            def infer(self, img, mask):
                return PatchStack(super().infer(img, mask).data * img.chroma)

        roiset = RoiSetWithDerivedChannels.from_binary_mask(
            self.stack,
            self.seg_mask,
            params=RoiSetMetaParams(
                filters={'area': {'min': 1e3, 'max': 1e4}},
                deproject_channel=0,
            )
        )
        self.assertIsInstance(roiset, RoiSetWithDerivedChannels)
        roiset.classify_by(
            'multiple_input_model',
            [0, 1],
            ModelWithDerivedInputs(),
            derived_channel_functions=[
                lambda acc: PatchStack(2 * acc.get_channels([0]).data),
                lambda acc: PatchStack((0.5 * acc.get_channels([1]).data).astype('uint8'))
            ]
        )
        self.assertTrue(all(roiset.get_df()['classify_by_multiple_input_model'].unique() == [4]))
        self.assertTrue(all(np.unique(roiset.get_object_class_map('multiple_input_model').data) == [0, 4]))

        self.assertEqual(len(roiset.accs_derived), 2)
        for di in roiset.accs_derived:
            self.assertEqual(roiset.get_patches_acc().hw, di.hw)
            self.assertEqual(roiset.get_patches_acc().nz, di.nz)
            self.assertEqual(roiset.get_patches_acc().count, di.count)

        dpas = roiset.run_exports(
            output_path / 'derived_channels', 0, 'der',
            RoiSetWithDerivedChannelsExportParams(derived_channels=True)
        )
        for fp in dpas['derived_channels']:
            assert Path(fp).exists()
        return roiset